@@ -3,12 +3,12 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 function test() {
-  let tab1 = gBrowser.addTab("about:rights");
-  let tab2 = gBrowser.addTab("about:mozilla");
-  whenBrowserLoaded(tab1.linkedBrowser, mainPart);
   waitForExplicitFinish();
 
-  function mainPart() {
+  let tab1 = gBrowser.addTab("about:rights");
+  let tab2 = gBrowser.addTab("about:mozilla");
+
+  promiseBrowserLoaded(tab1.linkedBrowser).then(() => {
     // Tell the session storer that the tab is pinned
     let newTabState = '{"entries":[{"url":"about:rights"}],"pinned":true,"userTypedValue":"Hello World!"}';
     ss.setTabState(tab1, newTabState);
@@ -26,5 +26,5 @@ function test() {
     gBrowser.removeTab(tab1);
     gBrowser.removeTab(tab2);
     finish();
-  }
+  });
 }
