@@ -15,27 +15,33 @@ BEGIN_FMRADIO_NAMESPACE
 
 class PFMRadioRequestParent;
 
-class FMRadioParent MOZ_FINAL : public PFMRadioParent
+class FMRadioParent final : public PFMRadioParent
                               , public FMRadioEventObserver
 {
 public:
   FMRadioParent();
   ~FMRadioParent();
 
+  virtual void
+  ActorDestroy(ActorDestroyReason aWhy) override;
+
   virtual bool
-  RecvGetStatusInfo(StatusInfo* aStatusInfo) MOZ_OVERRIDE;
+  RecvGetStatusInfo(StatusInfo* aStatusInfo) override;
 
   virtual PFMRadioRequestParent*
-  AllocPFMRadioRequestParent(const FMRadioRequestArgs& aArgs) MOZ_OVERRIDE;
+  AllocPFMRadioRequestParent(const FMRadioRequestArgs& aArgs) override;
 
   virtual bool
-  DeallocPFMRadioRequestParent(PFMRadioRequestParent* aActor) MOZ_OVERRIDE;
+  DeallocPFMRadioRequestParent(PFMRadioRequestParent* aActor) override;
 
   /* FMRadioEventObserver */
-  virtual void Notify(const FMRadioEventType& aType) MOZ_OVERRIDE;
+  virtual void Notify(const FMRadioEventType& aType) override;
 
   virtual bool
-  RecvEnableAudio(const bool& aAudioEnabled) MOZ_OVERRIDE;
+  RecvEnableAudio(const bool& aAudioEnabled) override;
+
+  virtual bool
+  RecvSetRDSGroupMask(const uint32_t& aRDSGroupMask) override;
 };
 
 END_FMRADIO_NAMESPACE
