@@ -18,7 +18,7 @@ public:
     virtual ~TestUrgencyParent();
 
     static bool RunTestInProcesses() { return true; }
-    static bool RunTestInThreads() { return true; }
+    static bool RunTestInThreads() { return false; }
 
     void Main();
 
@@ -29,14 +29,12 @@ public:
     bool RecvTest4_NestedSync();
     bool RecvFinalTest_Begin();
 
-    bool ShouldContinueFromReplyTimeout() MOZ_OVERRIDE
+    bool ShouldContinueFromReplyTimeout() override
     {
       return false;
     }
-    virtual void ActorDestroy(ActorDestroyReason why) MOZ_OVERRIDE
+    virtual void ActorDestroy(ActorDestroyReason why) override
     {
-        if (AbnormalShutdown != why)
-            fail("unexpected destruction!");  
         passed("ok");
         QuitParent();
     }
@@ -54,15 +52,11 @@ public:
     virtual ~TestUrgencyChild();
 
     bool RecvStart();
-    bool AnswerReply1(uint32_t *reply);
-    bool AnswerReply2(uint32_t *reply);
-    bool AnswerTest4_Reenter();
-    bool AnswerFinalTest_Hang();
+    bool RecvReply1(uint32_t *reply);
+    bool RecvReply2(uint32_t *reply);
 
-    virtual void ActorDestroy(ActorDestroyReason why) MOZ_OVERRIDE
+    virtual void ActorDestroy(ActorDestroyReason why) override
     {
-        if (AbnormalShutdown != why)
-            fail("unexpected destruction!");
         QuitChild();
     }
 
