@@ -18,7 +18,7 @@
 #include "jscompartment.h"
 
 #include "jit/CompileInfo.h"
-#include "jit/IonAllocPolicy.h"
+#include "jit/JitAllocPolicy.h"
 #include "jit/JitCompartment.h"
 #ifdef JS_ION_PERF
 # include "jit/PerfSpewer.h"
@@ -50,7 +50,7 @@ class MIRGenerator
         return alloc().ensureBallast();
     }
     const JitRuntime* jitRuntime() const {
-        return GetIonContext()->runtime->jitRuntime();
+        return GetJitContext()->runtime->jitRuntime();
     }
     CompileInfo& info() {
         return *info_;
@@ -60,10 +60,11 @@ class MIRGenerator
     }
 
     template <typename T>
-    T * allocate(size_t count = 1) {
-        if (count & mozilla::tl::MulOverflowMask<sizeof(T)>::value)
+    T* allocate(size_t count = 1) {
+        size_t bytes;
+        if (MOZ_UNLIKELY(!CalculateAllocSize<T>(count, &bytes)))
             return nullptr;
-        return reinterpret_cast<T*>(alloc().allocate(sizeof(T) * count));
+        return static_cast<T*>(alloc().allocate(bytes));
     }
 
     // Set an error state and prints a message. Returns false so errors can be
@@ -76,33 +77,61 @@ class MIRGenerator
     }
 
     bool instrumentedProfiling() {
-        return GetIonContext()->runtime->spsProfiler().enabled();
+        if (!instrumentedProfilingIsCached_) {
+            instrumentedProfiling_ = GetJitContext()->runtime->spsProfiler().enabled();
+            instrumentedProfilingIsCached_ = true;
+        }
+        return instrumentedProfiling_;
+    }
+
+    bool isProfilerInstrumentationEnabled() {
+        return !compilingAsmJS() && instrumentedProfiling();
+    }
+
+    bool isOptimizationTrackingEnabled() {
+        return isProfilerInstrumentationEnabled() && !info().isAnalysis();
     }
 
     // Whether the main thread is trying to cancel this build.
     bool shouldCancel(const char* why) {
+        maybePause();
         return cancelBuild_;
     }
     void cancel() {
         cancelBuild_ = true;
     }
 
+    void maybePause() {
+        if (pauseBuild_ && *pauseBuild_)
+            PauseCurrentHelperThread();
+    }
+    void setPauseFlag(mozilla::Atomic<bool, mozilla::Relaxed>* pauseBuild) {
+        pauseBuild_ = pauseBuild;
+    }
+
+    void disable() {
+        abortReason_ = AbortReason_Disable;
+    }
+    AbortReason abortReason() {
+        return abortReason_;
+    }
+
     bool compilingAsmJS() const {
-        return info_->script() == nullptr;
+        return info_->compilingAsmJS();
     }
 
     uint32_t maxAsmJSStackArgBytes() const {
-        JS_ASSERT(compilingAsmJS());
+        MOZ_ASSERT(compilingAsmJS());
         return maxAsmJSStackArgBytes_;
     }
     uint32_t resetAsmJSMaxStackArgBytes() {
-        JS_ASSERT(compilingAsmJS());
+        MOZ_ASSERT(compilingAsmJS());
         uint32_t old = maxAsmJSStackArgBytes_;
         maxAsmJSStackArgBytes_ = 0;
         return old;
     }
     void setAsmJSMaxStackArgBytes(uint32_t n) {
-        JS_ASSERT(compilingAsmJS());
+        MOZ_ASSERT(compilingAsmJS());
         maxAsmJSStackArgBytes_ = n;
     }
     void setPerformsCall() {
@@ -111,16 +140,11 @@ class MIRGenerator
     bool performsCall() const {
         return performsCall_;
     }
-    void setPerformsAsmJSCall() {
-        JS_ASSERT(compilingAsmJS());
-        setPerformsCall();
-        performsAsmJSCall_ = true;
-    }
-    bool performsAsmJSCall() const {
-        JS_ASSERT(compilingAsmJS());
-        return performsAsmJSCall_;
-    }
-    void noteMinAsmJSHeapLength(uint32_t len) {
+    // Traverses the graph to find if there's any SIMD instruction. Costful but
+    // the value is cached, so don't worry about calling it several times.
+    bool usesSimd();
+    void initMinAsmJSHeapLength(uint32_t len) {
+        MOZ_ASSERT(minAsmJSHeapLength_ == 0);
         minAsmJSHeapLength_ = len;
     }
     uint32_t minAsmJSHeapLength() const {
@@ -131,6 +155,14 @@ class MIRGenerator
         return modifiesFrameArguments_;
     }
 
+    typedef Vector<ObjectGroup*, 0, JitAllocPolicy> ObjectGroupVector;
+
+    // When abortReason() == AbortReason_NewScriptProperties, all types which
+    // the new script properties analysis hasn't been performed on yet.
+    const ObjectGroupVector& abortedNewScriptPropertiesGroups() const {
+        return abortedNewScriptPropertiesGroups_;
+    }
+
   public:
     CompileCompartment* compartment;
 
@@ -141,12 +173,17 @@ class MIRGenerator
     JSFunction* fun_;
     uint32_t nslots_;
     MIRGraph* graph_;
+    AbortReason abortReason_;
+    bool shouldForceAbort_; // Force AbortReason_Disable
+    ObjectGroupVector abortedNewScriptPropertiesGroups_;
     bool error_;
+    mozilla::Atomic<bool, mozilla::Relaxed>* pauseBuild_;
     mozilla::Atomic<bool, mozilla::Relaxed> cancelBuild_;
 
     uint32_t maxAsmJSStackArgBytes_;
     bool performsCall_;
-    bool performsAsmJSCall_;
+    bool usesSimd_;
+    bool usesSimdCached_;
     uint32_t minAsmJSHeapLength_;
 
     // Keep track of whether frame arguments are modified during execution.
@@ -154,6 +191,23 @@ class MIRGenerator
     // slots is not compatible with that.
     bool modifiesFrameArguments_;
 
+    bool instrumentedProfiling_;
+    bool instrumentedProfilingIsCached_;
+
+    // List of nursery objects used by this compilation. Can be traced by a
+    // minor GC while compilation happens off-thread. This Vector should only
+    // be accessed on the main thread (IonBuilder, nursery GC or
+    // CodeGenerator::link).
+    ObjectVector nurseryObjects_;
+
+    void addAbortedNewScriptPropertiesGroup(ObjectGroup* type);
+    void setForceAbort() {
+        shouldForceAbort_ = true;
+    }
+    bool shouldForceAbort() {
+        return shouldForceAbort_;
+    }
+
 #if defined(JS_ION_PERF)
     AsmJSPerfSpewer asmJSPerfSpewer_;
 
@@ -163,6 +217,12 @@ class MIRGenerator
 
   public:
     const JitCompileOptions options;
+
+    void traceNurseryObjects(JSTracer* trc);
+
+    const ObjectVector& nurseryObjects() const {
+        return nurseryObjects_;
+    }
 };
 
 } // namespace jit
