@@ -1,11 +1,11 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- tab-width: 2; indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 
 /**
-   File Name:          15.8-3.js
+   File Name:          15.8-2.js
    ECMA Section:       15.8 The Math Object
 
    Description:
@@ -29,19 +29,20 @@
    Date:               12 november 1997
 
 */
-var SECTION = "15.8-3-n";
+
+var SECTION = "15.8-2-n";
 var VERSION = "ECMA_1";
 startTest();
 var TITLE   = "The Math Object";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-DESCRIPTION = "MYMATH = Math()";
+DESCRIPTION = "MYMATH = new Math()";
 EXPECTED = "error";
 
 new TestCase( SECTION,
-	      "MYMATH = Math()",
+	      "MYMATH = new Math()",
 	      "error",
-	      eval("MYMATH = Math()") );
+	      eval("MYMATH = new Math()") );
 
 test();
