@@ -4,25 +4,29 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 334807;
-var summary = '10.1.8 - arguments prototype is the original Object prototype.';
-var actual = 'No Error';
-var expect = 'TypeError';
+var BUGNUMBER = 360969;
+var summary = '2^17: global function';
+var actual = 'No Crash';
+var expect = 'No Crash';
+
+var global = this;
 
 printBugNumber(BUGNUMBER);
 printStatus (summary);
+ 
+var start = new Date();
+var p;
+var i;
+var limit = 2 << 16;
 
-printStatus('set Object = Array');
-
-Object = Array;
-
-try
+for (var i = 0; i < limit; i++)
 {
-  (function () { printStatus( arguments.join()); })( 1, 2, 3 ); 
-}
-catch(ex)
-{
-  printStatus(ex + '');
-  actual = ex.name;
+  eval('function pf' + i + '() {}');
 }
+
 reportCompare(expect, actual, summary);
+
+var stop = new Date();
+
+print('Elapsed time: ' + Math.floor((stop - start)/1000) + ' seconds');
+
