@@ -4,8 +4,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 352870;
-var summary = 'Do not assert for crazy huge testcases';
+var BUGNUMBER = 465460;
+var summary = 'TM: valueOf in a loop: do not assert';
 var actual = '';
 var expect = '';
 
@@ -20,17 +20,13 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
  
-  expect = /TypeError: .+\.g (has no properties|is undefined)/;
-  actual = '';
-  try
-  {
-    switch(4) { case [(let (y = [].j(5)) ({}))
-                      for (p in ([1,2,3,4].g).v({},((w).y(z, [1]))))]: }  }
-  catch(ex)
-  {
-    actual = ex + '';
-  }
-  reportMatch(expect, actual, summary);
+  jit (true);
+
+  for each (let j in [null, 2, 3]) { }
+
+  jit(false);
+
+  reportCompare(expect, actual, summary);
 
   exitFunc ('test');
 }
