@@ -4,14 +4,13 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch08/8.12/8.12.1/8.12.1-1_47.js
- * @description Properties - [[HasOwnProperty]] (non-configurable, enumerable inherited getter/setter property)
+ * @path ch08/8.12/8.12.1/8.12.1-1_24.js
+ * @description Properties - [[HasOwnProperty]] (literal inherited setter property)
  */
 
 function testcase() {
 
-    var base = {};
-    Object.defineProperty(base, "foo", {get: function() {return 42;}, set: function() {;}, enumerable:true});
+    var base = { set foo(x) {;} };
     var o = Object.create(base);
     return o.hasOwnProperty("foo")===false;
 
