@@ -4,19 +4,19 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.4/10.4.3/10.4.3-1-31-s.js
- * @description Strict Mode - checking 'this' (FunctionExpression defined within a FunctionExpression inside strict mode)
- * @onlyStrict
+ * @path ch10/10.1/10.1.1/10.1.1-30-s.js
+ * @description Strict Mode - Function code of built-in Function constructor contains Use Strict Directive which appears at the start of the block
+ * @noStrict
  */
-    
+
+
 function testcase() {
-"use strict";
-var f1 = function () {
-    var f = function () {
-        return typeof this;
+        try {
+            var funObj = new Function("a", "'use strict'; eval('public = 1;');");
+            funObj();
+            return false;
+        } catch (e) {
+            return e instanceof SyntaxError;
+        }
     }
-    return (f()==="undefined") && ((typeof this)==="undefined");
-}
-return f1();
-}
-runTestCase(testcase);
\ No newline at end of file
+runTestCase(testcase);
