@@ -4,21 +4,20 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch11/11.13/11.13.1/11.13.1-4-31-s.js
- * @description Strict Mode - SyntaxError is thrown if the identifier 'arguments' appears as the LeftHandSideExpression (PrimaryExpression) of simple assignment(=) under strict mode
+ * @path ch11/11.1/11.1.5/11.1.5-2-s.js
+ * @description Strict Mode - SyntaxError is thrown when 'arguments' occurs as the Identifier in a PropertySetParameterList of a PropertyAssignment that is contained in strict code
  * @onlyStrict
  */
 
 
-
 function testcase() {
-        "use strict";
-        var blah = arguments;
+         "use strict";
+
         try {
-            eval("(arguments) = 20;");
-            return false;
+            eval("var obj = {set _11_1_5_2_fun(arguments) {} };");
+             return false;
         } catch (e) {
-            return e instanceof SyntaxError && blah === arguments;
+            return (e instanceof SyntaxError);
         }
-}
-runTestCase(testcase);
\ No newline at end of file
+    }
+runTestCase(testcase);
