@@ -8,11 +8,10 @@
 #include "jsapi.h"
 #include "jsfriendapi.h"
 #include "jsprf.h"
-#include "js/OldDebugAPI.h"
+#include "mozilla/dom/ScriptSettings.h"
 #include "nsServiceManagerUtils.h"
 #include "nsComponentManagerUtils.h"
 #include "nsIXPConnect.h"
-#include "nsIJSNativeInitializer.h"
 #include "nsIServiceManager.h"
 #include "nsIFile.h"
 #include "nsString.h"
@@ -29,9 +28,11 @@
 #include "nsJSPrincipals.h"
 #include "xpcpublic.h"
 #include "BackstagePass.h"
-#include "nsCxPusher.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIPrincipal.h"
+#include "nsJSUtils.h"
+
+#include "base/histogram.h"
 
 #ifdef ANDROID
 #include <android/log.h>
@@ -70,8 +71,9 @@ public:
     ~XPCShellDirProvider() { }
 
     // The platform resource folder
-    void SetGREDir(nsIFile* greDir);
-    void ClearGREDir() { mGREDir = nullptr; }
+    void SetGREDirs(nsIFile* greDir);
+    void ClearGREDirs() { mGREDir = nullptr;
+                          mGREBinDir = nullptr; }
     // The application resource folder
     void SetAppDir(nsIFile* appFile);
     void ClearAppDir() { mAppDir = nullptr; }
@@ -84,19 +86,12 @@ public:
 
 private:
     nsCOMPtr<nsIFile> mGREDir;
+    nsCOMPtr<nsIFile> mGREBinDir;
     nsCOMPtr<nsIFile> mAppDir;
     nsCOMPtr<nsIFile> mPluginDir;
     nsCOMPtr<nsIFile> mAppFile;
 };
 
-#ifdef JS_THREADSAFE
-#define DoBeginRequest(cx) JS_BeginRequest((cx))
-#define DoEndRequest(cx)   JS_EndRequest((cx))
-#else
-#define DoBeginRequest(cx) ((void)0)
-#define DoEndRequest(cx)   ((void)0)
-#endif
-
 static const char kXPConnectServiceContractID[] = "@mozilla.org/js/xpc/XPConnect;1";
 
 #define EXITCODE_RUNTIME_ERROR 3
@@ -116,8 +111,13 @@ static JSPrincipals* gJSPrincipals = nullptr;
 static nsAutoString* gWorkingDirectory = nullptr;
 
 static bool
-GetLocationProperty(JSContext* cx, HandleObject obj, HandleId id, MutableHandleValue vp)
+GetLocationProperty(JSContext* cx, unsigned argc, Value* vp)
 {
+    CallArgs args = CallArgsFromVp(argc, vp);
+    if (!args.thisv().isObject()) {
+        JS_ReportError(cx, "Unexpected this value for GetLocationProperty");
+        return false;
+    }
 #if !defined(XP_WIN) && !defined(XP_UNIX)
     //XXX: your platform should really implement this
     return false;
@@ -178,13 +178,13 @@ GetLocationProperty(JSContext* cx, HandleObject obj, HandleId id, MutableHandleV
             if (NS_SUCCEEDED(location->IsSymlink(&symlink)) &&
                 !symlink)
                 location->Normalize();
-            rv = xpc->WrapNative(cx, obj, location,
+            rv = xpc->WrapNative(cx, &args.thisv().toObject(), location,
                                  NS_GET_IID(nsIFile),
                                  getter_AddRefs(locationHolder));
 
             if (NS_SUCCEEDED(rv) &&
                 locationHolder->GetJSObject()) {
-                vp.set(OBJECT_TO_JSVAL(locationHolder->GetJSObject()));
+                args.rval().setObject(*locationHolder->GetJSObject());
             }
         }
     }
@@ -196,7 +196,7 @@ GetLocationProperty(JSContext* cx, HandleObject obj, HandleId id, MutableHandleV
 static bool
 GetLine(JSContext* cx, char* bufp, FILE* file, const char* prompt) {
     {
-        char line[256] = { '\0' };
+        char line[4096] = { '\0' };
         fputs(prompt, gOutFile);
         fflush(gOutFile);
         if ((!fgets(line, sizeof line, file) && errno != EINTR) || feof(file))
@@ -257,26 +257,23 @@ Print(JSContext* cx, unsigned argc, jsval* vp)
     args.rval().setUndefined();
 
     RootedString str(cx);
-    nsAutoCString utf8str;
-    size_t length;
-    const jschar* chars;
+    nsAutoCString utf8output;
 
     for (unsigned i = 0; i < args.length(); i++) {
         str = ToString(cx, args[i]);
         if (!str)
             return false;
-        chars = JS_GetStringCharsAndLength(cx, str, &length);
-        if (!chars)
+
+        JSAutoByteString utf8str;
+        if (!utf8str.encodeUtf8(cx, str))
             return false;
 
         if (i)
-            utf8str.Append(' ');
-        AppendUTF16toUTF8(Substring(reinterpret_cast<const char16_t*>(chars),
-                                    length),
-                          utf8str);
+            utf8output.Append(' ');
+        utf8output.Append(utf8str.ptr(), utf8str.length());
     }
-    utf8str.Append('\n');
-    fputs(utf8str.get(), gOutFile);
+    utf8output.Append('\n');
+    fputs(utf8output.get(), gOutFile);
     fflush(gOutFile);
     return true;
 }
@@ -294,22 +291,22 @@ Dump(JSContext* cx, unsigned argc, jsval* vp)
     if (!str)
         return false;
 
-    size_t length;
-    const jschar* chars = JS_GetStringCharsAndLength(cx, str, &length);
-    if (!chars)
+    JSAutoByteString utf8str;
+    if (!utf8str.encodeUtf8(cx, str))
         return false;
 
-    NS_ConvertUTF16toUTF8 utf8str(reinterpret_cast<const char16_t*>(chars),
-                                  length);
 #ifdef ANDROID
-    __android_log_print(ANDROID_LOG_INFO, "Gecko", "%s", utf8str.get());
+    __android_log_print(ANDROID_LOG_INFO, "Gecko", "%s", utf8str.ptr());
 #endif
 #ifdef XP_WIN
     if (IsDebuggerPresent()) {
-      OutputDebugStringW(reinterpret_cast<const wchar_t*>(chars));
+        nsAutoJSString wstr;
+        if (!wstr.init(cx, str))
+            return false;
+        OutputDebugStringW(wstr.get());
     }
 #endif
-    fputs(utf8str.get(), gOutFile);
+    fputs(utf8str.ptr(), gOutFile);
     fflush(gOutFile);
     return true;
 }
@@ -339,8 +336,10 @@ Load(JSContext* cx, unsigned argc, jsval* vp)
         }
         JS::CompileOptions options(cx);
         options.setUTF8(true)
-               .setFileAndLine(filename.ptr(), 1);
-        JS::Rooted<JSScript*> script(cx, JS::Compile(cx, obj, options, file));
+               .setFileAndLine(filename.ptr(), 1)
+               .setCompileAndGo(true);
+        JS::Rooted<JSScript*> script(cx);
+        JS::Compile(cx, obj, options, file, &script);
         fclose(file);
         if (!script)
             return false;
@@ -375,8 +374,11 @@ BuildDate(JSContext* cx, unsigned argc, jsval* vp)
 static bool
 Quit(JSContext* cx, unsigned argc, jsval* vp)
 {
+    CallArgs args = CallArgsFromVp(argc, vp);
+
     gExitCode = 0;
-    JS_ConvertArguments(cx, JS::CallArgsFromVp(argc, vp),"/ i", &gExitCode);
+    if (!ToInt32(cx, args.get(0), &gExitCode))
+        return false;
 
     gQuitting = true;
 //    exit(0);
@@ -454,7 +456,7 @@ SendCommand(JSContext* cx, unsigned argc, Value* vp)
         return false;
     }
 
-    JSString* str = ToString(cx, args[0]);
+    RootedString str(cx, ToString(cx, args[0]));
     if (!str) {
         JS_ReportError(cx, "Could not convert argument 1 to string!");
         return false;
@@ -478,7 +480,7 @@ static bool
 Options(JSContext* cx, unsigned argc, jsval* vp)
 {
     JS::CallArgs args = CallArgsFromVp(argc, vp);
-    ContextOptions oldOptions = ContextOptionsRef(cx);
+    RuntimeOptions oldRuntimeOptions = RuntimeOptionsRef(cx);
 
     for (unsigned i = 0; i < args.length(); ++i) {
         JSString* str = ToString(cx, args[i]);
@@ -490,11 +492,11 @@ Options(JSContext* cx, unsigned argc, jsval* vp)
             return false;
 
         if (strcmp(opt.ptr(), "strict") == 0)
-            ContextOptionsRef(cx).toggleExtraWarnings();
+            RuntimeOptionsRef(cx).toggleExtraWarnings();
         else if (strcmp(opt.ptr(), "werror") == 0)
-            ContextOptionsRef(cx).toggleWerror();
+            RuntimeOptionsRef(cx).toggleWerror();
         else if (strcmp(opt.ptr(), "strict_mode") == 0)
-            ContextOptionsRef(cx).toggleStrictMode();
+            RuntimeOptionsRef(cx).toggleStrictMode();
         else {
             JS_ReportError(cx, "unknown option name '%s'. The valid names are "
                            "strict, werror, and strict_mode.", opt.ptr());
@@ -503,21 +505,21 @@ Options(JSContext* cx, unsigned argc, jsval* vp)
     }
 
     char* names = nullptr;
-    if (oldOptions.extraWarnings()) {
+    if (oldRuntimeOptions.extraWarnings()) {
         names = JS_sprintf_append(names, "%s", "strict");
         if (!names) {
             JS_ReportOutOfMemory(cx);
             return false;
         }
     }
-    if (oldOptions.werror()) {
+    if (oldRuntimeOptions.werror()) {
         names = JS_sprintf_append(names, "%s%s", names ? "," : "", "werror");
         if (!names) {
             JS_ReportOutOfMemory(cx);
             return false;
         }
     }
-    if (names && oldOptions.strictMode()) {
+    if (names && oldRuntimeOptions.strictMode()) {
         names = JS_sprintf_append(names, "%s%s", names ? "," : "", "strict_mode");
         if (!names) {
             JS_ReportOutOfMemory(cx);
@@ -544,7 +546,7 @@ Parent(JSContext* cx, unsigned argc, jsval* vp)
     }
 
     Value v = args[0];
-    if (JSVAL_IS_PRIMITIVE(v)) {
+    if (v.isPrimitive()) {
         JS_ReportError(cx, "Only objects have parents!");
         return false;
     }
@@ -573,91 +575,13 @@ Btoa(JSContext* cx, unsigned argc, Value* vp)
   return xpc::Base64Encode(cx, args[0], args.rval());
 }
 
-static bool
-Blob(JSContext* cx, unsigned argc, Value* vp)
-{
-  JS::CallArgs args = CallArgsFromVp(argc, vp);
-
-  nsCOMPtr<nsISupports> native =
-    do_CreateInstance("@mozilla.org/dom/multipart-blob;1");
-  if (!native) {
-    JS_ReportError(cx, "Could not create native object!");
-    return false;
-  }
-
-  nsCOMPtr<nsIJSNativeInitializer> initializer = do_QueryInterface(native);
-  MOZ_ASSERT(initializer);
-
-  nsresult rv = initializer->Initialize(nullptr, cx, nullptr, args);
-  if (NS_FAILED(rv)) {
-    JS_ReportError(cx, "Could not initialize native object!");
-    return false;
-  }
-
-  nsCOMPtr<nsIXPConnect> xpc = do_GetService(kXPConnectServiceContractID, &rv);
-  if (NS_FAILED(rv)) {
-    JS_ReportError(cx, "Could not get XPConnent service!");
-    return false;
-  }
-
-  JSObject* global = JS::CurrentGlobalOrNull(cx);
-  rv = xpc->WrapNativeToJSVal(cx, global, native, nullptr,
-                              &NS_GET_IID(nsISupports), true,
-                              args.rval());
-  if (NS_FAILED(rv)) {
-    JS_ReportError(cx, "Could not wrap native object!");
-    return false;
-  }
-
-  return true;
-}
-
-static bool
-File(JSContext* cx, unsigned argc, Value* vp)
-{
-  JS::CallArgs args = CallArgsFromVp(argc, vp);
-
-  nsCOMPtr<nsISupports> native =
-    do_CreateInstance("@mozilla.org/dom/multipart-file;1");
-  if (!native) {
-    JS_ReportError(cx, "Could not create native object!");
-    return false;
-  }
-
-  nsCOMPtr<nsIJSNativeInitializer> initializer = do_QueryInterface(native);
-  MOZ_ASSERT(initializer);
-
-  nsresult rv = initializer->Initialize(nullptr, cx, nullptr, args);
-  if (NS_FAILED(rv)) {
-    JS_ReportError(cx, "Could not initialize native object!");
-    return false;
-  }
-
-  nsCOMPtr<nsIXPConnect> xpc = do_GetService(kXPConnectServiceContractID, &rv);
-  if (NS_FAILED(rv)) {
-    JS_ReportError(cx, "Could not get XPConnent service!");
-    return false;
-  }
-
-  JSObject* global = JS::CurrentGlobalOrNull(cx);
-  rv = xpc->WrapNativeToJSVal(cx, global, native, nullptr,
-                              &NS_GET_IID(nsISupports), true,
-                              args.rval());
-  if (NS_FAILED(rv)) {
-    JS_ReportError(cx, "Could not wrap native object!");
-    return false;
-  }
-
-  return true;
-}
-
-static Maybe<PersistentRootedValue> sScriptedInterruptCallback;
+static PersistentRootedValue sScriptedInterruptCallback;
 
 static bool
 XPCShellInterruptCallback(JSContext* cx)
 {
-    MOZ_ASSERT(!sScriptedInterruptCallback.empty());
-    RootedValue callback(cx, sScriptedInterruptCallback.ref());
+    MOZ_ASSERT(sScriptedInterruptCallback.initialized());
+    RootedValue callback(cx, sScriptedInterruptCallback);
 
     // If no interrupt callback was set by script, no-op.
     if (callback.isUndefined())
@@ -679,7 +603,7 @@ XPCShellInterruptCallback(JSContext* cx)
 static bool
 SetInterruptCallback(JSContext* cx, unsigned argc, jsval* vp)
 {
-    MOZ_ASSERT(!sScriptedInterruptCallback.empty());
+    MOZ_ASSERT(sScriptedInterruptCallback.initialized());
 
     // Sanity-check args.
     JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
@@ -690,17 +614,17 @@ SetInterruptCallback(JSContext* cx, unsigned argc, jsval* vp)
 
     // Allow callers to remove the interrupt callback by passing undefined.
     if (args[0].isUndefined()) {
-        sScriptedInterruptCallback.ref() = UndefinedValue();
+        sScriptedInterruptCallback = UndefinedValue();
         return true;
     }
 
     // Otherwise, we should have a callable object.
-    if (!args[0].isObject() || !JS_ObjectIsCallable(cx, &args[0].toObject())) {
+    if (!args[0].isObject() || !JS::IsCallable(&args[0].toObject())) {
         JS_ReportError(cx, "Argument must be callable");
         return false;
     }
 
-    sScriptedInterruptCallback.ref() = args[0];
+    sScriptedInterruptCallback = args[0];
 
     return true;
 }
@@ -737,8 +661,6 @@ static const JSFunctionSpec glob_functions[] = {
     JS_FS("sendCommand",     SendCommand,    1,0),
     JS_FS("atob",            Atob,           1,0),
     JS_FS("btoa",            Btoa,           1,0),
-    JS_FS("Blob",            Blob,           2,JSFUN_CONSTRUCTOR),
-    JS_FS("File",            File,           2,JSFUN_CONSTRUCTOR),
     JS_FS("setInterruptCallback", SetInterruptCallback, 1,0),
     JS_FS("simulateActivityCallback", SimulateActivityCallback, 1,0),
     JS_FS_END
@@ -826,10 +748,9 @@ env_enumerate(JSContext* cx, HandleObject obj)
 }
 
 static bool
-env_resolve(JSContext* cx, HandleObject obj, HandleId id,
-            JS::MutableHandleObject objp)
+env_resolve(JSContext* cx, HandleObject obj, HandleId id, bool* resolvedp)
 {
-    JSString* idstr, *valstr;
+    JSString* idstr;
 
     RootedValue idval(cx);
     if (!JS_IdToValue(cx, id, &idval))
@@ -843,24 +764,21 @@ env_resolve(JSContext* cx, HandleObject obj, HandleId id,
         return false;
     const char* value = getenv(name.ptr());
     if (value) {
-        valstr = JS_NewStringCopyZ(cx, value);
+        RootedString valstr(cx, JS_NewStringCopyZ(cx, value));
         if (!valstr)
             return false;
-        if (!JS_DefinePropertyById(cx, obj, id, STRING_TO_JSVAL(valstr),
-                                   nullptr, nullptr, JSPROP_ENUMERATE)) {
+        if (!JS_DefinePropertyById(cx, obj, id, valstr, JSPROP_ENUMERATE)) {
             return false;
         }
-        objp.set(obj);
+        *resolvedp = true;
     }
     return true;
 }
 
 static const JSClass env_class = {
-    "environment", JSCLASS_HAS_PRIVATE | JSCLASS_NEW_RESOLVE,
-    JS_PropertyStub,  JS_DeletePropertyStub,
-    JS_PropertyStub,  env_setProperty,
-    env_enumerate, (JSResolveOp) env_resolve,
-    JS_ConvertStub,   nullptr
+    "environment", JSCLASS_HAS_PRIVATE,
+    nullptr, nullptr, nullptr, env_setProperty,
+    env_enumerate, env_resolve
 };
 
 /***************************************************************************/
@@ -881,7 +799,7 @@ static const JSErrorFormatString jsShell_ErrorFormatString[JSShellErr_Limit] = {
 };
 
 static const JSErrorFormatString*
-my_GetErrorMessage(void* userRef, const char* locale, const unsigned errorNumber)
+my_GetErrorMessage(void* userRef, const unsigned errorNumber)
 {
     if (errorNumber == 0 || errorNumber >= JSShellErr_Limit)
         return nullptr;
@@ -920,15 +838,15 @@ ProcessFile(JSContext* cx, JS::Handle<JSObject*> obj, const char* filename, FILE
             }
         }
         ungetc(ch, file);
-        DoBeginRequest(cx);
+        JS_BeginRequest(cx);
 
         JS::CompileOptions options(cx);
         options.setUTF8(true)
-               .setFileAndLine(filename, 1);
-        script = JS::Compile(cx, obj, options, file);
-        if (script && !compileOnly)
+               .setFileAndLine(filename, 1)
+               .setCompileAndGo(true);
+        if (JS::Compile(cx, obj, options, file, &script) && !compileOnly)
             (void)JS_ExecuteScript(cx, obj, script, &result);
-        DoEndRequest(cx);
+        JS_EndRequest(cx);
 
         return;
     }
@@ -956,22 +874,22 @@ ProcessFile(JSContext* cx, JS::Handle<JSObject*> obj, const char* filename, FILE
             lineno++;
         } while (!JS_BufferIsCompilableUnit(cx, obj, buffer, strlen(buffer)));
 
-        DoBeginRequest(cx);
+        JS_BeginRequest(cx);
         /* Clear any pending exception from previous failed compiles.  */
         JS_ClearPendingException(cx);
         JS::CompileOptions options(cx);
-        options.setFileAndLine("typein", startline);
-        script = JS_CompileScript(cx, obj, buffer, strlen(buffer), options);
-        if (script) {
+        options.setFileAndLine("typein", startline)
+               .setCompileAndGo(true);
+        if (JS_CompileScript(cx, obj, buffer, strlen(buffer), options, &script)) {
             JSErrorReporter older;
 
             if (!compileOnly) {
                 ok = JS_ExecuteScript(cx, obj, script, &result);
                 if (ok && result != JSVAL_VOID) {
                     /* Suppress error reports from JS::ToString(). */
-                    older = JS_SetErrorReporter(cx, nullptr);
+                    older = JS_SetErrorReporter(JS_GetRuntime(cx), nullptr);
                     str = ToString(cx, result);
-                    JS_SetErrorReporter(cx, older);
+                    JS_SetErrorReporter(JS_GetRuntime(cx), older);
                     JSAutoByteString bytes;
                     if (str && bytes.encodeLatin1(cx, str))
                         fprintf(gOutFile, "%s\n", bytes.ptr());
@@ -980,7 +898,7 @@ ProcessFile(JSContext* cx, JS::Handle<JSObject*> obj, const char* filename, FILE
                 }
             }
         }
-        DoEndRequest(cx);
+        JS_EndRequest(cx);
     } while (!hitEOF && !gQuitting);
 
     fprintf(gOutFile, "\n");
@@ -1013,7 +931,7 @@ static int
 usage(void)
 {
     fprintf(gErrFile, "%s\n", JS_GetImplementationVersion());
-    fprintf(gErrFile, "usage: xpcshell [-g gredir] [-a appdir] [-r manifest]... [-PsSwWCijmIn] [-v version] [-f scriptfile] [-e script] [scriptfile] [scriptarg...]\n");
+    fprintf(gErrFile, "usage: xpcshell [-g gredir] [-a appdir] [-r manifest]... [-WwxiCSsmIp] [-v version] [-f scriptfile] [-e script] [scriptfile] [scriptarg...]\n");
     return 2;
 }
 
@@ -1032,9 +950,9 @@ ProcessArgsForCompartment(JSContext* cx, char** argv, int argc)
                 return;
             break;
         case 'S':
-            ContextOptionsRef(cx).toggleWerror();
+            RuntimeOptionsRef(cx).toggleWerror();
         case 's':
-            ContextOptionsRef(cx).toggleExtraWarnings();
+            RuntimeOptionsRef(cx).toggleExtraWarnings();
             break;
         case 'I':
             RuntimeOptionsRef(cx).toggleIon()
@@ -1094,11 +1012,9 @@ ProcessArgs(JSContext* cx, JS::Handle<JSObject*> obj, char** argv, int argc, XPC
         return 1;
 
     for (size_t j = 0, length = argc - i; j < length; j++) {
-        JSString* str = JS_NewStringCopyZ(cx, argv[i++]);
-        if (!str)
-            return 1;
-        if (!JS_DefineElement(cx, argsObj, j, STRING_TO_JSVAL(str),
-                              nullptr, nullptr, JSPROP_ENUMERATE)) {
+        RootedString str(cx, JS_NewStringCopyZ(cx, argv[i++]));
+        if (!str ||
+            !JS_DefineElement(cx, argsObj, j, str, JSPROP_ENUMERATE)) {
             return 1;
         }
     }
@@ -1126,7 +1042,9 @@ ProcessArgs(JSContext* cx, JS::Handle<JSObject*> obj, char** argv, int argc, XPC
         case 'x':
             break;
         case 'd':
-            xpc_ActivateDebugMode();
+            /* This used to try to turn on the debugger. */
+            break;
+        case 'm':
             break;
         case 'f':
             if (++i == argc) {
@@ -1151,7 +1069,9 @@ ProcessArgs(JSContext* cx, JS::Handle<JSObject*> obj, char** argv, int argc, XPC
                 return usage();
             }
 
-            JS_EvaluateScript(cx, obj, argv[i], strlen(argv[i]), "-e", 1, &rval);
+            JS::CompileOptions opts(cx);
+            opts.setFileAndLine("-e", 1);
+            JS::Evaluate(cx, obj, opts, argv[i], strlen(argv[i]), &rval);
 
             isInteractive = false;
             break;
@@ -1162,7 +1082,6 @@ ProcessArgs(JSContext* cx, JS::Handle<JSObject*> obj, char** argv, int argc, XPC
             break;
         case 'S':
         case 's':
-        case 'm':
         case 'I':
             // These options are processed in ProcessArgsForCompartment.
             break;
@@ -1275,14 +1194,6 @@ XPCShellErrorReporter(JSContext* cx, const char* message, JSErrorReport* rep)
     xpc::SystemErrorReporter(cx, message, rep);
 }
 
-static bool
-ContextCallback(JSContext* cx, unsigned contextOp)
-{
-    if (contextOp == JSCONTEXT_NEW)
-        JS_SetErrorReporter(cx, XPCShellErrorReporter);
-    return true;
-}
-
 static bool
 GetCurrentWorkingDirectory(nsAString& workingDirectory)
 {
@@ -1336,6 +1247,11 @@ XRE_XPCShellMain(int argc, char** argv, char** envp)
 
     NS_LogInit();
 
+    // A initializer to initialize histogram collection
+    // used by telemetry.
+    UniquePtr<base::StatisticsRecorder> telStats =
+       MakeUnique<base::StatisticsRecorder>();
+
     nsCOMPtr<nsIFile> appFile;
     rv = XRE_GetBinaryPath(argv[0], getter_AddRefs(appFile));
     if (NS_FAILED(rv)) {
@@ -1364,11 +1280,27 @@ XRE_XPCShellMain(int argc, char** argv, char** envp)
             return 1;
         }
 
-        dirprovider.SetGREDir(greDir);
+        dirprovider.SetGREDirs(greDir);
 
         argc -= 2;
         argv += 2;
     } else {
+#ifdef XP_MACOSX
+        // On OSX, the GreD needs to point to Contents/Resources in the .app
+        // bundle. Libraries will be loaded at a relative path to GreD, i.e.
+        // ../MacOS.
+        nsCOMPtr<nsIFile> tmpDir;
+        XRE_GetFileFromPath(argv[0], getter_AddRefs(greDir));
+        greDir->GetParent(getter_AddRefs(tmpDir));
+        tmpDir->Clone(getter_AddRefs(greDir));
+        tmpDir->SetNativeLeafName(NS_LITERAL_CSTRING("Resources"));
+        bool dirExists = false;
+        tmpDir->Exists(&dirExists);
+        if (dirExists) {
+            greDir = tmpDir.forget();
+        }
+        dirprovider.SetGREDirs(greDir);
+#else
         nsAutoString workingDir;
         if (!GetCurrentWorkingDirectory(workingDir)) {
             printf("GetCurrentWorkingDirectory failed.\n");
@@ -1379,6 +1311,7 @@ XRE_XPCShellMain(int argc, char** argv, char** envp)
             printf("NS_NewLocalFile failed.\n");
             return 1;
         }
+#endif
     }
 
     if (argc > 1 && !strcmp(argv[1], "-a")) {
@@ -1464,19 +1397,17 @@ XRE_XPCShellMain(int argc, char** argv, char** envp)
             return 1;
         }
 
-        rtsvc->RegisterContextCallback(ContextCallback);
-
         // Override the default XPConnect interrupt callback. We could store the
         // old one and restore it before shutting down, but there's not really a
         // reason to bother.
-        sScriptedInterruptCallback.construct(rt, UndefinedValue());
+        sScriptedInterruptCallback.init(rt, UndefinedValue());
         JS_SetInterruptCallback(rt, XPCShellInterruptCallback);
 
-        cx = JS_NewContext(rt, 8192);
-        if (!cx) {
-            printf("JS_NewContext failed!\n");
-            return 1;
-        }
+        JS_SetErrorReporter(rt, XPCShellErrorReporter);
+
+        dom::AutoJSAPI jsapi;
+        jsapi.Init();
+        cx = jsapi.cx();
 
         argc--;
         argv++;
@@ -1521,9 +1452,6 @@ XRE_XPCShellMain(int argc, char** argv, char** envp)
         xpc->SetFunctionThisTranslator(NS_GET_IID(nsITestXPCFunctionCallback), translator);
 #endif
 
-        nsCxPusher pusher;
-        pusher.Push(cx);
-
         nsRefPtr<BackstagePass> backstagePass;
         rv = NS_NewBackstagePass(getter_AddRefs(backstagePass));
         if (NS_FAILED(rv)) {
@@ -1563,20 +1491,17 @@ XRE_XPCShellMain(int argc, char** argv, char** envp)
             JSAutoCompartment ac(cx, glob);
 
             if (!JS_InitReflect(cx, glob)) {
-                JS_EndRequest(cx);
                 return 1;
             }
 
             if (!JS_DefineFunctions(cx, glob, glob_functions) ||
                 !JS_DefineProfilingFunctions(cx, glob)) {
-                JS_EndRequest(cx);
                 return 1;
             }
 
             JS::Rooted<JSObject*> envobj(cx);
-            envobj = JS_DefineObject(cx, glob, "environment", &env_class, nullptr, 0);
+            envobj = JS_DefineObject(cx, glob, "environment", &env_class);
             if (!envobj) {
-                JS_EndRequest(cx);
                 return 1;
             }
 
@@ -1586,18 +1511,21 @@ XRE_XPCShellMain(int argc, char** argv, char** envp)
             if (GetCurrentWorkingDirectory(workingDirectory))
                 gWorkingDirectory = &workingDirectory;
 
-            JS_DefineProperty(cx, glob, "__LOCATION__", JS::UndefinedHandleValue, 0,
-                              GetLocationProperty, nullptr);
+            JS_DefineProperty(cx, glob, "__LOCATION__", JS::UndefinedHandleValue,
+                              JSPROP_SHARED,
+                              GetLocationProperty,
+                              nullptr);
 
-            result = ProcessArgs(cx, glob, argv, argc, &dirprovider);
+            // We are almost certainly going to run script here, so we need an
+            // AutoEntryScript. This is Gecko-specific and not in any spec.
+            dom::AutoEntryScript aes(backstagePass);
+            result = ProcessArgs(aes.cx(), glob, argv, argc, &dirprovider);
 
             JS_DropPrincipals(rt, gJSPrincipals);
-            JS_SetAllNonReservedSlotsToUndefined(cx, glob);
+            JS_SetAllNonReservedSlotsToUndefined(aes.cx(), glob);
             JS_GC(rt);
         }
-        pusher.Pop();
         JS_GC(rt);
-        JS_DestroyContext(cx);
     } // this scopes the nsCOMPtrs
 
     if (!XRE_ShutdownTestShell())
@@ -1607,8 +1535,6 @@ XRE_XPCShellMain(int argc, char** argv, char** envp)
     rv = NS_ShutdownXPCOM( nullptr );
     MOZ_ASSERT(NS_SUCCEEDED(rv), "NS_ShutdownXPCOM failed");
 
-    sScriptedInterruptCallback.destroyIfConstructed();
-
 #ifdef TEST_CALL_ON_WRAPPED_JS_AFTER_SHUTDOWN
     // test of late call and release (see above)
     JSContext* bogusCX;
@@ -1616,9 +1542,10 @@ XRE_XPCShellMain(int argc, char** argv, char** envp)
     bogus = nullptr;
 #endif
 
+    telStats = nullptr;
     appDir = nullptr;
     appFile = nullptr;
-    dirprovider.ClearGREDir();
+    dirprovider.ClearGREDirs();
     dirprovider.ClearAppDir();
     dirprovider.ClearPluginDir();
     dirprovider.ClearAppFile();
@@ -1635,9 +1562,17 @@ XRE_XPCShellMain(int argc, char** argv, char** envp)
 }
 
 void
-XPCShellDirProvider::SetGREDir(nsIFile* greDir)
+XPCShellDirProvider::SetGREDirs(nsIFile* greDir)
 {
     mGREDir = greDir;
+    mGREDir->Clone(getter_AddRefs(mGREBinDir));
+#ifdef XP_MACOSX
+    nsAutoCString leafName;
+    mGREDir->GetNativeLeafName(leafName);
+    if (leafName.Equals("Resources")) {
+        mGREBinDir->SetNativeLeafName(NS_LITERAL_CSTRING("MacOS"));
+    }
+#endif
 }
 
 void
@@ -1681,6 +1616,9 @@ XPCShellDirProvider::GetFile(const char* prop, bool* persistent,
     if (mGREDir && !strcmp(prop, NS_GRE_DIR)) {
         *persistent = true;
         return mGREDir->Clone(result);
+    } else if (mGREBinDir && !strcmp(prop, NS_GRE_BIN_DIR)) {
+        *persistent = true;
+        return mGREBinDir->Clone(result);
     } else if (mAppFile && !strcmp(prop, XRE_EXECUTABLE_FILE)) {
         *persistent = true;
         return mAppFile->Clone(result);
