@@ -4,6 +4,11 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsMathMLmencloseFrame.h"
+
+#include "gfx2DGlue.h"
+#include "gfxUtils.h"
+#include "mozilla/gfx/2D.h"
+#include "mozilla/gfx/PathHelpers.h"
 #include "nsPresContext.h"
 #include "nsRenderingContext.h"
 #include "nsWhitespaceTokenizer.h"
@@ -13,6 +18,9 @@
 #include "nsMathMLChar.h"
 #include <algorithm>
 
+using namespace mozilla;
+using namespace mozilla::gfx;
+
 //
 // <menclose> -- enclose content with a stretching symbol such
 // as a long division sign. - implementation
@@ -28,6 +36,9 @@ static const char16_t kRadicalChar = 0x221A;
 // updiagonalstrike
 static const uint8_t kArrowHeadSize = 10;
 
+// phasorangle
+static const uint8_t kPhasorangleWidth = 8;
+
 nsIFrame*
 NS_NewMathMLmencloseFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
@@ -122,6 +133,8 @@ nsresult nsMathMLmencloseFrame::AddNotation(const nsAString& aNotation)
     mNotationsToDraw |= NOTATION_HORIZONTALSTRIKE;
   } else if (aNotation.EqualsLiteral("madruwb")) {
     mNotationsToDraw |= (NOTATION_RIGHT | NOTATION_BOTTOM);
+  } else if (aNotation.EqualsLiteral("phasorangle")) {
+    mNotationsToDraw |= (NOTATION_BOTTOM | NOTATION_PHASORANGLE);
   }
 
   return NS_OK;
@@ -207,10 +220,15 @@ nsMathMLmencloseFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
     nsRect rect;
     mMathMLChar[mRadicalCharIndex].GetRect(rect);
     rect.MoveBy(StyleVisibility()->mDirection ? -mContentWidth : rect.width, 0);
-    rect.SizeTo(mContentWidth, mRuleThickness);
+    rect.SizeTo(mContentWidth, mRadicalRuleThickness);
     DisplayBar(aBuilder, this, rect, aLists);
   }
 
+  if (IsToDraw(NOTATION_PHASORANGLE)) {
+    DisplayNotation(aBuilder, this, mencloseRect, aLists,
+                mRuleThickness, NOTATION_PHASORANGLE);
+  }
+
   if (IsToDraw(NOTATION_LONGDIV)) {
     mMathMLChar[mLongDivCharIndex].Display(aBuilder, this, aLists, 1);
 
@@ -319,20 +337,24 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
   nscoord radicalAscent = 0, radicalDescent = 0;
   nscoord longdivAscent = 0, longdivDescent = 0;
   nscoord psi = 0;
+  nscoord leading = 0;
 
   ///////////////
   // Thickness of bars and font metrics
   nscoord onePixel = nsPresContext::CSSPixelsToAppUnits(1);
 
-  nscoord mEmHeight;
+  float fontSizeInflation = nsLayoutUtils::FontSizeInflationFor(this);
   nsRefPtr<nsFontMetrics> fm;
-  nsLayoutUtils::GetFontMetricsForFrame(this, getter_AddRefs(fm));
-  aRenderingContext.SetFont(fm);
+  nsLayoutUtils::GetFontMetricsForFrame(this, getter_AddRefs(fm),
+                                        fontSizeInflation);
   GetRuleThickness(aRenderingContext, fm, mRuleThickness);
-  GetEmHeight(fm, mEmHeight);
+  if (mRuleThickness < onePixel) {
+    mRuleThickness = onePixel;
+  }
 
   char16_t one = '1';
-  nsBoundingMetrics bmOne = aRenderingContext.GetBoundingMetrics(&one, 1);
+  nsBoundingMetrics bmOne =
+    nsLayoutUtils::AppUnitBoundsOfString(&one, 1, *fm, aRenderingContext);
 
   ///////////////
   // General rules: the menclose element takes the size of the enclosed content.
@@ -345,23 +367,23 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
     padding += onePixel - delta; // round up
 
   if (IsToDraw(NOTATION_LONGDIV) || IsToDraw(NOTATION_RADICAL)) {
-      nscoord phi;
-      // Rule 11, App. G, TeXbook
-      // psi = clearance between rule and content
-      if (StyleFont()->mMathDisplay == NS_MATHML_DISPLAYSTYLE_BLOCK)
-        phi = fm->XHeight();
-      else
-        phi = mRuleThickness;
-      psi = mRuleThickness + phi / 4;
+    GetRadicalParameters(fm, StyleFont()->mMathDisplay ==
+                         NS_MATHML_DISPLAYSTYLE_BLOCK,
+                         mRadicalRuleThickness, leading, psi);
 
-      delta = psi % onePixel;
-      if (delta)
-        psi += onePixel - delta; // round up
+    // make sure that the rule appears on on screen
+    if (mRadicalRuleThickness < onePixel) {
+      mRadicalRuleThickness = onePixel;
     }
 
-  if (mRuleThickness < onePixel)
-    mRuleThickness = onePixel;
- 
+    // adjust clearance psi to get an exact number of pixels -- this
+    // gives a nicer & uniform look on stacked radicals (bug 130282)
+    delta = psi % onePixel;
+    if (delta) {
+      psi += onePixel - delta; // round up
+    }
+  }
+
   // Set horizontal parameters
   if (IsToDraw(NOTATION_ROUNDEDBOX) ||
       IsToDraw(NOTATION_TOP) ||
@@ -387,7 +409,8 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
       IsToDraw(NOTATION_CIRCLE) ||
       IsToDraw(NOTATION_ROUNDEDBOX) ||
       IsToDraw(NOTATION_RADICAL) ||
-      IsToDraw(NOTATION_LONGDIV)) {
+      IsToDraw(NOTATION_LONGDIV) ||
+      IsToDraw(NOTATION_PHASORANGLE)) {
       // set a minimal value for the base height
       bmBase.ascent = std::max(bmOne.ascent, bmBase.ascent);
       bmBase.descent = std::max(0, bmBase.descent);
@@ -410,6 +433,14 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
       IsToDraw(NOTATION_CIRCLE))
     mBoundingMetrics.descent += padding;
 
+   ///////////////
+   // phasorangle notation
+  if (IsToDraw(NOTATION_PHASORANGLE)) {
+    nscoord phasorangleWidth = kPhasorangleWidth * mRuleThickness;
+    // Update horizontal parameters
+    dx_left = std::max(dx_left, phasorangleWidth);
+  }
+
   ///////////////
   // updiagonal arrow notation. We need enough space at the top right corner to
   // draw the arrow head.
@@ -453,7 +484,7 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
   if (IsToDraw(NOTATION_LONGDIV)) {
     if (aWidthOnly) {
         nscoord longdiv_width = mMathMLChar[mLongDivCharIndex].
-          GetMaxWidth(PresContext(), aRenderingContext);
+          GetMaxWidth(PresContext(), aRenderingContext, fontSizeInflation);
 
         // Update horizontal parameters
         dx_left = std::max(dx_left, longdiv_width);
@@ -466,6 +497,7 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
 
       // height(longdiv) should be >= height(base) + psi + mRuleThickness
       mMathMLChar[mLongDivCharIndex].Stretch(PresContext(), aRenderingContext,
+                                             fontSizeInflation,
                                              NS_STRETCH_DIRECTION_VERTICAL,
                                              contSize, bmLongdivChar,
                                              NS_STRETCH_LARGER, false);
@@ -494,7 +526,7 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
     
     if (aWidthOnly) {
       nscoord radical_width = mMathMLChar[mRadicalCharIndex].
-        GetMaxWidth(PresContext(), aRenderingContext);
+        GetMaxWidth(PresContext(), aRenderingContext, fontSizeInflation);
       
       // Update horizontal parameters
       *dx_leading = std::max(*dx_leading, radical_width);
@@ -502,11 +534,12 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
       // Stretch the radical symbol to the appropriate height if it is not
       // big enough.
       nsBoundingMetrics contSize = bmBase;
-      contSize.ascent = mRuleThickness;
+      contSize.ascent = mRadicalRuleThickness;
       contSize.descent = bmBase.ascent + bmBase.descent + psi;
 
-      // height(radical) should be >= height(base) + psi + mRuleThickness
+      // height(radical) should be >= height(base) + psi + mRadicalRuleThickness
       mMathMLChar[mRadicalCharIndex].Stretch(PresContext(), aRenderingContext,
+                                             fontSizeInflation,
                                              NS_STRETCH_DIRECTION_VERTICAL,
                                              contSize, bmRadicalChar,
                                              NS_STRETCH_LARGER,
@@ -517,7 +550,7 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
       *dx_leading = std::max(*dx_leading, bmRadicalChar.width);
 
       // Update vertical parameters
-      radicalAscent = bmBase.ascent + psi + mRuleThickness;
+      radicalAscent = bmBase.ascent + psi + mRadicalRuleThickness;
       radicalDescent = std::max(bmBase.descent,
                               (bmRadicalChar.ascent + bmRadicalChar.descent -
                                radicalAscent));
@@ -548,17 +581,16 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
   
   aDesiredSize.Width() = mBoundingMetrics.width;
 
-  aDesiredSize.SetTopAscent(std::max(mBoundingMetrics.ascent, baseSize.TopAscent()));
-  aDesiredSize.Height() = aDesiredSize.TopAscent() +
-    std::max(mBoundingMetrics.descent, baseSize.Height() - baseSize.TopAscent());
+  aDesiredSize.SetBlockStartAscent(std::max(mBoundingMetrics.ascent,
+                                            baseSize.BlockStartAscent()));
+  aDesiredSize.Height() = aDesiredSize.BlockStartAscent() +
+    std::max(mBoundingMetrics.descent,
+             baseSize.Height() - baseSize.BlockStartAscent());
 
   if (IsToDraw(NOTATION_LONGDIV) || IsToDraw(NOTATION_RADICAL)) {
-    // get the leading to be left at the top of the resulting frame
-    // this seems more reliable than using fm->GetLeading() on suspicious
-    // fonts
-    nscoord leading = nscoord(0.2f * mEmHeight);
-    nscoord desiredSizeAscent = aDesiredSize.TopAscent();
-    nscoord desiredSizeDescent = aDesiredSize.Height() - aDesiredSize.TopAscent();
+    nscoord desiredSizeAscent = aDesiredSize.BlockStartAscent();
+    nscoord desiredSizeDescent = aDesiredSize.Height() -
+                                 aDesiredSize.BlockStartAscent();
     
     if (IsToDraw(NOTATION_LONGDIV)) {
       desiredSizeAscent = std::max(desiredSizeAscent,
@@ -571,10 +603,10 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
       desiredSizeAscent = std::max(desiredSizeAscent,
                                  radicalAscent + leading);
       desiredSizeDescent = std::max(desiredSizeDescent,
-                                  radicalDescent + mRuleThickness);
+                                    radicalDescent + mRadicalRuleThickness);
     }
 
-    aDesiredSize.SetTopAscent(desiredSizeAscent);
+    aDesiredSize.SetBlockStartAscent(desiredSizeAscent);
     aDesiredSize.Height() = desiredSizeAscent + desiredSizeDescent;
   }
     
@@ -582,12 +614,12 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
       IsToDraw(NOTATION_ROUNDEDBOX) ||
       (IsToDraw(NOTATION_TOP) && IsToDraw(NOTATION_BOTTOM))) {
     // center the menclose around the content (vertically)
-    nscoord dy = std::max(aDesiredSize.TopAscent() - bmBase.ascent,
-                        aDesiredSize.Height() - aDesiredSize.TopAscent() -
-                        bmBase.descent);
+    nscoord dy = std::max(aDesiredSize.BlockStartAscent() - bmBase.ascent,
+                          aDesiredSize.Height() -
+                          aDesiredSize.BlockStartAscent() - bmBase.descent);
 
-    aDesiredSize.SetTopAscent(bmBase.ascent + dy);
-    aDesiredSize.Height() = aDesiredSize.TopAscent() + bmBase.descent + dy;
+    aDesiredSize.SetBlockStartAscent(bmBase.ascent + dy);
+    aDesiredSize.Height() = aDesiredSize.BlockStartAscent() + bmBase.descent + dy;
   }
 
   // Update mBoundingMetrics ascent/descent
@@ -600,7 +632,7 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
       IsToDraw(NOTATION_VERTICALSTRIKE) ||
       IsToDraw(NOTATION_CIRCLE) ||
       IsToDraw(NOTATION_ROUNDEDBOX))
-    mBoundingMetrics.ascent = aDesiredSize.TopAscent();
+    mBoundingMetrics.ascent = aDesiredSize.BlockStartAscent();
   
   if (IsToDraw(NOTATION_BOTTOM) ||
       IsToDraw(NOTATION_RIGHT) ||
@@ -611,12 +643,17 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
       IsToDraw(NOTATION_VERTICALSTRIKE) ||
       IsToDraw(NOTATION_CIRCLE) ||
       IsToDraw(NOTATION_ROUNDEDBOX))
-    mBoundingMetrics.descent = aDesiredSize.Height() - aDesiredSize.TopAscent();
+    mBoundingMetrics.descent = aDesiredSize.Height() - aDesiredSize.BlockStartAscent();
 
+  // phasorangle notation:
+  // move up from the bottom by the angled line height
+  if (IsToDraw(NOTATION_PHASORANGLE))
+    mBoundingMetrics.ascent = std::max(mBoundingMetrics.ascent, 2 * kPhasorangleWidth * mRuleThickness - mBoundingMetrics.descent);
+  
   aDesiredSize.mBoundingMetrics = mBoundingMetrics;
   
   mReference.x = 0;
-  mReference.y = aDesiredSize.TopAscent();
+  mReference.y = aDesiredSize.BlockStartAscent();
 
   if (aPlaceOrigin) {
     //////////////////
@@ -624,7 +661,7 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
     if (IsToDraw(NOTATION_LONGDIV))
       mMathMLChar[mLongDivCharIndex].SetRect(nsRect(dx_left -
                                                     bmLongdivChar.width,
-                                                    aDesiredSize.TopAscent() -
+                                                    aDesiredSize.BlockStartAscent() -
                                                     longdivAscent,
                                                     bmLongdivChar.width,
                                                     bmLongdivChar.ascent +
@@ -635,7 +672,7 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
                     dx_left + bmBase.width : dx_left - bmRadicalChar.width);
 
       mMathMLChar[mRadicalCharIndex].SetRect(nsRect(dx,
-                                                    aDesiredSize.TopAscent() -
+                                                    aDesiredSize.BlockStartAscent() -
                                                     radicalAscent,
                                                     bmRadicalChar.width,
                                                     bmRadicalChar.ascent +
@@ -646,7 +683,7 @@ nsMathMLmencloseFrame::PlaceInternal(nsRenderingContext& aRenderingContext,
 
     //////////////////
     // Finish reflowing child frames
-    PositionRowChildFrames(dx_left, aDesiredSize.TopAscent());
+    PositionRowChildFrames(dx_left, aDesiredSize.BlockStartAscent());
   }
 
   return NS_OK;
@@ -722,7 +759,7 @@ public:
 #endif
 
   virtual void Paint(nsDisplayListBuilder* aBuilder,
-                     nsRenderingContext* aCtx) MOZ_OVERRIDE;
+                     nsRenderingContext* aCtx) override;
   NS_DISPLAY_DECL_NAME("MathMLMencloseNotation", TYPE_MATHML_MENCLOSE_NOTATION)
 
 private:
@@ -734,80 +771,86 @@ private:
 void nsDisplayNotation::Paint(nsDisplayListBuilder* aBuilder,
                               nsRenderingContext* aCtx)
 {
-  // get the gfxRect
+  DrawTarget& aDrawTarget = *aCtx->GetDrawTarget();
   nsPresContext* presContext = mFrame->PresContext();
-  gfxRect rect = presContext->AppUnitsToGfxUnits(mRect + ToReferenceFrame());
 
-  // paint the frame with the current text color
-  aCtx->SetColor(mFrame->GetVisitedDependentColor(eCSSProperty_color));
+  Float strokeWidth = presContext->AppUnitsToGfxUnits(mThickness);
 
-  // change line width to mThickness
-  gfxContext *gfxCtx = aCtx->ThebesContext();
-  gfxFloat e = presContext->AppUnitsToGfxUnits(mThickness);
-  gfxCtx->Save();
-  gfxCtx->SetLineWidth(e);
+  Rect rect = NSRectToRect(mRect + ToReferenceFrame(),
+                           presContext->AppUnitsPerDevPixel());
+  rect.Deflate(strokeWidth / 2.f);
 
-  rect.Deflate(e / 2.0);
+  ColorPattern color(ToDeviceColor(
+                       mFrame->GetVisitedDependentColor(eCSSProperty_color)));
+
+  StrokeOptions strokeOptions(strokeWidth);
 
   switch(mType)
-    {
-    case NOTATION_CIRCLE:
-      gfxCtx->NewPath();
-      gfxCtx->Ellipse(rect.Center(), rect.Size());
-      gfxCtx->Stroke();
-      break;
-
-    case NOTATION_ROUNDEDBOX:
-      gfxCtx->NewPath();
-      gfxCtx->RoundedRectangle(rect, gfxCornerSizes(3 * e), true);
-      gfxCtx->Stroke();
-      break;
-
-    case NOTATION_UPDIAGONALSTRIKE:
-      gfxCtx->NewPath();
-      gfxCtx->Line(rect.BottomLeft(), rect.TopRight());
-      gfxCtx->Stroke();
-      break;
-
-    case NOTATION_DOWNDIAGONALSTRIKE:
-      gfxCtx->NewPath();
-      gfxCtx->Line(rect.TopLeft(), rect.BottomRight());
-      gfxCtx->Stroke();
-      break;
-
+  {
+    case NOTATION_CIRCLE: {
+      RefPtr<Path> ellipse =
+        MakePathForEllipse(aDrawTarget, rect.Center(), rect.Size());
+      aDrawTarget.Stroke(ellipse, color, strokeOptions);
+      return;
+    }
+    case NOTATION_ROUNDEDBOX: {
+      Float radius = 3 * strokeWidth;
+      RectCornerRadii radii(radius, radius);
+      RefPtr<Path> roundedRect =
+        MakePathForRoundedRect(aDrawTarget, rect, radii, true);
+      aDrawTarget.Stroke(roundedRect, color, strokeOptions);
+      return;
+    }
+    case NOTATION_UPDIAGONALSTRIKE: {
+      aDrawTarget.StrokeLine(rect.BottomLeft(), rect.TopRight(),
+                             color, strokeOptions);
+      return;
+    }
+    case NOTATION_DOWNDIAGONALSTRIKE: {
+      aDrawTarget.StrokeLine(rect.TopLeft(), rect.BottomRight(),
+                             color, strokeOptions);
+      return;
+    }
     case NOTATION_UPDIAGONALARROW: {
       // Compute some parameters to draw the updiagonalarrow. The values below
       // are taken from MathJax's HTML-CSS output.
-      gfxFloat W = rect.Width(); gfxFloat H = rect.Height();
-      gfxFloat l = sqrt(W*W + H*H);
-      gfxFloat f = gfxFloat(kArrowHeadSize) * e / l;
-      gfxFloat w = W * f; gfxFloat h = H * f;
+      Float W = rect.Width(); gfxFloat H = rect.Height();
+      Float l = sqrt(W*W + H*H);
+      Float f = Float(kArrowHeadSize) * strokeWidth / l;
+      Float w = W * f; gfxFloat h = H * f;
 
       // Draw the arrow shaft
-      gfxCtx->NewPath();
-      gfxCtx->Line(rect.BottomLeft(), rect.TopRight() + gfxPoint(-.7*w, .7*h));
-      gfxCtx->Stroke();
+      aDrawTarget.StrokeLine(rect.BottomLeft(),
+                             rect.TopRight() + Point(-.7*w, .7*h),
+                             color, strokeOptions);
 
       // Draw the arrow head
-      gfxCtx->NewPath();
-      gfxPoint p[] = {
-        rect.TopRight(),
-        rect.TopRight() + gfxPoint(-w -.4*h, std::max(-e / 2.0, h - .4*w)),
-        rect.TopRight() + gfxPoint(-.7*w, .7*h),
-        rect.TopRight() + gfxPoint(std::min(e / 2.0, -w + .4*h), h + .4*w),
-        rect.TopRight()
-      };
-      gfxCtx->Polygon(p, MOZ_ARRAY_LENGTH(p));
-      gfxCtx->Fill();
+      RefPtr<PathBuilder> builder = aDrawTarget.CreatePathBuilder();
+      builder->MoveTo(rect.TopRight());
+      builder->LineTo(rect.TopRight() + Point(-w -.4*h, std::max(-strokeWidth / 2.0, h - .4*w)));
+      builder->LineTo(rect.TopRight() + Point(-.7*w, .7*h));
+      builder->LineTo(rect.TopRight() + Point(std::min(strokeWidth / 2.0, -w + .4*h), h + .4*w));
+      builder->Close();
+      RefPtr<Path> path = builder->Finish();
+      aDrawTarget.Fill(path, color);
+      return;
     }
-      break;
+    case NOTATION_PHASORANGLE: {
+      // Compute some parameters to draw the angled line,
+      // that uses a slope of 2 (angle = tan^-1(2)).
+      // H = w * tan(angle) = w * 2
+      Float w = Float(kPhasorangleWidth) * strokeWidth;
+      Float H = 2 * w;
 
+      // Draw the angled line
+      aDrawTarget.StrokeLine(rect.BottomLeft(),
+                             rect.BottomLeft() + Point(w, -H),
+                             color, strokeOptions);
+      return;
+    }
     default:
       NS_NOTREACHED("This notation can not be drawn using nsDisplayNotation");
-      break;
-    }
-
-  gfxCtx->Restore();
+  }
 }
 
 void
