@@ -1,3 +1,7 @@
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
 package org.mozilla.gecko.tests;
 
 /**
@@ -8,7 +12,7 @@ package org.mozilla.gecko.tests;
  */
 public class testLoad extends PixelTest {
     public void testLoad() {
-        String url = getAbsoluteUrl("/robocop/robocop_boxes.html");
+        String url = getAbsoluteUrl(StringHelper.ROBOCOP_BOXES_URL);
 
         blockForGeckoReady();
 
