@@ -5,9 +5,9 @@
 #include "CacheStorage.h"
 #include "CacheStorageService.h"
 #include "LoadContextInfo.h"
+#include "nsCacheService.h"
 
 #include "nsIURI.h"
-#include "nsICacheService.h"
 #include "nsICacheSession.h"
 #include "nsIApplicationCache.h"
 #include "nsIApplicationCacheService.h"
@@ -19,6 +19,7 @@
 
 #include "nsServiceManagerUtils.h"
 #include "nsNetCID.h"
+#include "nsNetUtil.h"
 #include "nsProxyRelease.h"
 #include "mozilla/Telemetry.h"
 
@@ -38,7 +39,7 @@ namespace { // anon
 class DoomCallbackSynchronizer : public nsRunnable
 {
 public:
-  DoomCallbackSynchronizer(nsICacheEntryDoomCallback* cb) : mCB(cb)
+  explicit DoomCallbackSynchronizer(nsICacheEntryDoomCallback* cb) : mCB(cb)
   {
     MOZ_COUNT_CTOR(DoomCallbackSynchronizer);
   }
@@ -91,7 +92,7 @@ class DoomCallbackWrapper : public nsICacheListener
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSICACHELISTENER
 
-  DoomCallbackWrapper(nsICacheEntryDoomCallback* cb) : mCB(cb)
+  explicit DoomCallbackWrapper(nsICacheEntryDoomCallback* cb) : mCB(cb)
   {
     MOZ_COUNT_CTOR(DoomCallbackWrapper);
   }
@@ -124,43 +125,31 @@ NS_IMETHODIMP DoomCallbackWrapper::OnCacheEntryDoomed(nsresult status)
   return NS_OK;
 }
 
+} // anon
+
+// _OldVisitCallbackWrapper
 // Receives visit callbacks from the old API and forwards it to the new API
 
-class VisitCallbackWrapper : public nsICacheVisitor
-{
-  NS_DECL_THREADSAFE_ISUPPORTS
-  NS_DECL_NSICACHEVISITOR
+NS_IMPL_ISUPPORTS(_OldVisitCallbackWrapper, nsICacheVisitor)
 
-  VisitCallbackWrapper(char* const deviceID,
-                       nsICacheStorageVisitor* cb,
-                       bool visitEntries)
-  : mCB(cb)
-  , mVisitEntries(visitEntries)
-  , mDeviceID(deviceID)
-  {
-    MOZ_COUNT_CTOR(VisitCallbackWrapper);
+_OldVisitCallbackWrapper::~_OldVisitCallbackWrapper()
+{
+  if (!mHit) {
+    // The device has not been found, to not break the chain, simulate
+    // storage info callback.
+    mCB->OnCacheStorageInfo(0, 0, 0, nullptr);
   }
 
-private:
-  virtual ~VisitCallbackWrapper();
-  nsCOMPtr<nsICacheStorageVisitor> mCB;
-  bool mVisitEntries;
-  char* const mDeviceID;
-};
-
-NS_IMPL_ISUPPORTS(VisitCallbackWrapper, nsICacheVisitor)
-
-VisitCallbackWrapper::~VisitCallbackWrapper()
-{
-  if (mVisitEntries)
+  if (mVisitEntries) {
     mCB->OnCacheEntryVisitCompleted();
+  }
 
-  MOZ_COUNT_DTOR(VisitCallbackWrapper);
+  MOZ_COUNT_DTOR(_OldVisitCallbackWrapper);
 }
 
-NS_IMETHODIMP VisitCallbackWrapper::VisitDevice(const char * deviceID,
-                                                nsICacheDeviceInfo *deviceInfo,
-                                                bool *_retval)
+NS_IMETHODIMP _OldVisitCallbackWrapper::VisitDevice(const char * deviceID,
+                                                    nsICacheDeviceInfo *deviceInfo,
+                                                    bool *_retval)
 {
   if (!mCB)
     return NS_ERROR_NULL_POINTER;
@@ -171,38 +160,120 @@ NS_IMETHODIMP VisitCallbackWrapper::VisitDevice(const char * deviceID,
     return NS_OK;
   }
 
+  mHit = true;
+
   nsresult rv;
 
-  uint32_t entryCount;
-  rv = deviceInfo->GetEntryCount(&entryCount);
+  uint32_t capacity;
+  rv = deviceInfo->GetMaximumSize(&capacity);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  uint32_t totalSize;
-  rv = deviceInfo->GetTotalSize(&totalSize);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIFile> dir;
+  if (!strcmp(mDeviceID, "disk")) {
+    nsCacheService::GetDiskCacheDirectory(getter_AddRefs(dir));
+  } else if (!strcmp(mDeviceID, "offline")) {
+    nsCacheService::GetAppCacheDirectory(getter_AddRefs(dir));
+  }
+
+  if (mLoadInfo->IsAnonymous()) {
+    // Anonymous visiting reports 0, 0 since we cannot count that
+    // early the number of anon entries.
+    mCB->OnCacheStorageInfo(0, 0, capacity, dir);
+  } else {
+    // Non-anon visitor counts all non-anon + ALL ANON entries,
+    // there is no way to determine the number of entries when
+    // using the old cache APIs - there is no concept of anonymous
+    // storage.
+    uint32_t entryCount;
+    rv = deviceInfo->GetEntryCount(&entryCount);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    uint32_t totalSize;
+    rv = deviceInfo->GetTotalSize(&totalSize);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    mCB->OnCacheStorageInfo(entryCount, totalSize, capacity, dir);
+  }
 
-  mCB->OnCacheStorageInfo(entryCount, totalSize);
   *_retval = mVisitEntries;
-
   return NS_OK;
 }
 
-NS_IMETHODIMP VisitCallbackWrapper::VisitEntry(const char * deviceID,
-                                               nsICacheEntryInfo *entryInfo,
-                                               bool *_retval)
+NS_IMETHODIMP _OldVisitCallbackWrapper::VisitEntry(const char * deviceID,
+                                                   nsICacheEntryInfo *entryInfo,
+                                                   bool *_retval)
 {
   MOZ_ASSERT(!strcmp(deviceID, mDeviceID));
 
-  nsRefPtr<_OldCacheEntryWrapper> wrapper = new _OldCacheEntryWrapper(entryInfo);
-  nsresult rv = mCB->OnCacheEntryInfo(wrapper);
-  *_retval = NS_SUCCEEDED(rv);
+  nsresult rv;
 
+  *_retval = true;
+
+  // Read all informative properties from the entry.
+  nsXPIDLCString clientId;
+  rv = entryInfo->GetClientID(getter_Copies(clientId));
+  if (NS_FAILED(rv))
+    return NS_OK;
+
+  if (mLoadInfo->IsPrivate() !=
+      StringBeginsWith(clientId, NS_LITERAL_CSTRING("HTTP-memory-only-PB"))) {
+    return NS_OK;
+  }
+
+  nsAutoCString cacheKey, enhanceId;
+  rv = entryInfo->GetKey(cacheKey);
+  if (NS_FAILED(rv))
+    return NS_OK;
+
+  if (StringBeginsWith(cacheKey, NS_LITERAL_CSTRING("anon&"))) {
+    if (!mLoadInfo->IsAnonymous())
+      return NS_OK;
+
+    cacheKey = Substring(cacheKey, 5, cacheKey.Length());
+  } else if (mLoadInfo->IsAnonymous()) {
+    return NS_OK;
+  }
+
+  if (StringBeginsWith(cacheKey, NS_LITERAL_CSTRING("id="))) {
+    int32_t uriSpecEnd = cacheKey.Find("&uri=");
+    if (uriSpecEnd == kNotFound) // Corrupted, ignore
+      return NS_OK;
+
+    enhanceId = Substring(cacheKey, 3, uriSpecEnd - 3);
+    cacheKey = Substring(cacheKey, uriSpecEnd + 1, cacheKey.Length());
+  }
+
+  if (StringBeginsWith(cacheKey, NS_LITERAL_CSTRING("uri="))) {
+    cacheKey = Substring(cacheKey, 4, cacheKey.Length());
+  }
+
+  nsCOMPtr<nsIURI> uri;
+  // cacheKey is strip of any prefixes
+  rv = NS_NewURI(getter_AddRefs(uri), cacheKey);
+  if (NS_FAILED(rv))
+    return NS_OK;
+
+  uint32_t dataSize;
+  if (NS_FAILED(entryInfo->GetDataSize(&dataSize)))
+    dataSize = 0;
+  int32_t fetchCount;
+  if (NS_FAILED(entryInfo->GetFetchCount(&fetchCount)))
+    fetchCount = 0;
+  uint32_t expirationTime;
+  if (NS_FAILED(entryInfo->GetExpirationTime(&expirationTime)))
+    expirationTime = 0;
+  uint32_t lastModified;
+  if (NS_FAILED(entryInfo->GetLastModified(&lastModified)))
+    lastModified = 0;
+
+  // Send them to the consumer.
+  rv = mCB->OnCacheEntryInfo(
+    uri, enhanceId, (int64_t)dataSize, fetchCount, lastModified, expirationTime);
+
+  *_retval = NS_SUCCEEDED(rv);
   return NS_OK;
 }
 
-} // anon
-
-
 // _OldGetDiskConsumption
 
 //static
@@ -295,6 +366,18 @@ _OldCacheEntryWrapper::~_OldCacheEntryWrapper()
   LOG(("Destroying _OldCacheEntryWrapper %p for descriptor %p", this, mOldInfo.get()));
 }
 
+NS_IMETHODIMP _OldCacheEntryWrapper::GetIsForcedValid(bool *aIsForcedValid)
+{
+  // Unused stub
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP _OldCacheEntryWrapper::ForceValidFor(uint32_t aSecondsToTheFuture)
+{
+  // Unused stub
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMPL_ISUPPORTS(_OldCacheEntryWrapper, nsICacheEntry)
 
 NS_IMETHODIMP _OldCacheEntryWrapper::AsyncDoom(nsICacheEntryDoomCallback* listener)
@@ -407,6 +490,36 @@ NS_IMETHODIMP _OldCacheEntryWrapper::HasWriteAccess(bool aWriteAllowed_unused, b
   return NS_OK;
 }
 
+namespace { // anon
+
+class MetaDataVisitorWrapper : public nsICacheMetaDataVisitor
+{
+  virtual ~MetaDataVisitorWrapper() {}
+
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSICACHEMETADATAVISITOR
+  explicit MetaDataVisitorWrapper(nsICacheEntryMetaDataVisitor* cb) : mCB(cb) {}
+  nsCOMPtr<nsICacheEntryMetaDataVisitor> mCB;
+};
+
+NS_IMPL_ISUPPORTS(MetaDataVisitorWrapper, nsICacheMetaDataVisitor)
+
+NS_IMETHODIMP
+MetaDataVisitorWrapper::VisitMetaDataElement(char const * key,
+                                             char const * value,
+                                             bool *goon)
+{
+  *goon = true;
+  return mCB->OnMetaDataElement(key, value);
+}
+
+} // anon
+
+NS_IMETHODIMP _OldCacheEntryWrapper::VisitMetaData(nsICacheEntryMetaDataVisitor* cb)
+{
+  nsRefPtr<MetaDataVisitorWrapper> w = new MetaDataVisitorWrapper(cb);
+  return mOldDesc->VisitMetaData(w);
+}
 
 namespace { // anon
 
@@ -422,38 +535,38 @@ GetCacheSessionNameForStoragePolicy(
   MOZ_ASSERT(!isPrivate || storagePolicy == nsICache::STORE_IN_MEMORY);
 
   // HTTP
-  if (scheme.Equals(NS_LITERAL_CSTRING("http")) ||
-      scheme.Equals(NS_LITERAL_CSTRING("https"))) {
+  if (scheme.EqualsLiteral("http") ||
+      scheme.EqualsLiteral("https")) {
     switch (storagePolicy) {
     case nsICache::STORE_IN_MEMORY:
       if (isPrivate)
-        sessionName.Assign(NS_LITERAL_CSTRING("HTTP-memory-only-PB"));
+        sessionName.AssignLiteral("HTTP-memory-only-PB");
       else
-        sessionName.Assign(NS_LITERAL_CSTRING("HTTP-memory-only"));
+        sessionName.AssignLiteral("HTTP-memory-only");
       break;
     case nsICache::STORE_OFFLINE:
       // XXX This is actually never used, only added to prevent
       // any compatibility damage.
-      sessionName.Assign(NS_LITERAL_CSTRING("HTTP-offline"));
+      sessionName.AssignLiteral("HTTP-offline");
       break;
     default:
-      sessionName.Assign(NS_LITERAL_CSTRING("HTTP"));
+      sessionName.AssignLiteral("HTTP");
       break;
     }
   }
   // WYCIWYG
-  else if (scheme.Equals(NS_LITERAL_CSTRING("wyciwyg"))) {
+  else if (scheme.EqualsLiteral("wyciwyg")) {
     if (isPrivate)
-      sessionName.Assign(NS_LITERAL_CSTRING("wyciwyg-private"));
+      sessionName.AssignLiteral("wyciwyg-private");
     else
-      sessionName.Assign(NS_LITERAL_CSTRING("wyciwyg"));
+      sessionName.AssignLiteral("wyciwyg");
   }
   // FTP
-  else if (scheme.Equals(NS_LITERAL_CSTRING("ftp"))) {
+  else if (scheme.EqualsLiteral("ftp")) {
     if (isPrivate)
-      sessionName.Assign(NS_LITERAL_CSTRING("FTP-private"));
+      sessionName.AssignLiteral("FTP-private");
     else
-      sessionName.Assign(NS_LITERAL_CSTRING("FTP"));
+      sessionName.AssignLiteral("FTP");
   }
   // all remaining URL scheme
   else {
@@ -464,9 +577,9 @@ GetCacheSessionNameForStoragePolicy(
     // URL schemes.
     // Deliberately omitting |anonymous| since other session types don't
     // recognize it too.
-    sessionName.Assign(NS_LITERAL_CSTRING("other"));
+    sessionName.AssignLiteral("other");
     if (isPrivate)
-      sessionName.Append(NS_LITERAL_CSTRING("-private"));
+      sessionName.AppendLiteral("-private");
   }
 
   if (appId != nsILoadContextInfo::NO_APP_ID || inBrowser) {
@@ -518,10 +631,10 @@ GetCacheSession(nsCSubstring const &aScheme,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsICacheSession> session;
-  rv = serv->CreateSession(clientId.get(),
-                           storagePolicy,
-                           nsICache::STREAM_BASED,
-                           getter_AddRefs(session));
+  rv = nsCacheService::GlobalInstance()->CreateSessionInternal(clientId.get(),
+                                                               storagePolicy,
+                                                               nsICache::STREAM_BASED,
+                                                               getter_AddRefs(session));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = session->SetIsPrivate(aLoadInfo->IsPrivate());
@@ -843,6 +956,18 @@ NS_IMETHODIMP _OldStorage::AsyncOpenURI(nsIURI *aURI,
   return NS_OK;
 }
 
+NS_IMETHODIMP _OldStorage::OpenTruncate(nsIURI *aURI, const nsACString & aIdExtension,
+                                        nsICacheEntry **aCacheEntry)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP _OldStorage::Exists(nsIURI *aURI, const nsACString & aIdExtension,
+                                   bool *aResult)
+{
+  return NS_ERROR_NOT_AVAILABLE;
+}
+
 NS_IMETHODIMP _OldStorage::AsyncDoomURI(nsIURI *aURI, const nsACString & aIdExtension,
                                         nsICacheEntryDoomCallback* aCallback)
 {
@@ -884,7 +1009,7 @@ NS_IMETHODIMP _OldStorage::AsyncEvictStorage(nsICacheEntryDoomCallback* aCallbac
           do_GetService(NS_CACHESERVICE_CONTRACTID, &rv);
       NS_ENSURE_SUCCESS(rv, rv);
 
-      rv = serv->EvictEntries(nsICache::STORE_OFFLINE);
+      rv = nsCacheService::GlobalInstance()->EvictEntriesInternal(nsICache::STORE_OFFLINE);
       NS_ENSURE_SUCCESS(rv, rv);
     }
     else {
@@ -956,23 +1081,10 @@ NS_IMETHODIMP _OldStorage::AsyncVisitStorage(nsICacheStorageVisitor* aVisitor,
 
   NS_ENSURE_ARG(aVisitor);
 
-  if (mLoadInfo->IsAnonymous()) {
-    // There is no concept of 'anonymous' storage in the old cache
-    // since anon cache entries are stored in 'non-anon' storage
-    // with a special prefix.
-    // Just fake we have 0 items with 0 consumption.  This at least
-    // prevents displaying double size in the advanced section of
-    // the Options dialog.
-    aVisitor->OnCacheStorageInfo(0, 0);
-    if (aVisitEntries)
-      aVisitor->OnCacheEntryVisitCompleted();
-    return NS_OK;
-  }
-
   nsresult rv;
 
   nsCOMPtr<nsICacheService> serv =
-      do_GetService(NS_CACHESERVICE_CONTRACTID, &rv);
+    do_GetService(NS_CACHESERVICE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
   char* deviceID;
@@ -984,9 +1096,9 @@ NS_IMETHODIMP _OldStorage::AsyncVisitStorage(nsICacheStorageVisitor* aVisitor,
     deviceID = const_cast<char*>("disk");
   }
 
-  nsRefPtr<VisitCallbackWrapper> cb = new VisitCallbackWrapper(
-    deviceID, aVisitor, aVisitEntries);
-  rv = serv->VisitEntries(cb);
+  nsRefPtr<_OldVisitCallbackWrapper> cb = new _OldVisitCallbackWrapper(
+    deviceID, aVisitor, aVisitEntries, mLoadInfo);
+  rv = nsCacheService::GlobalInstance()->VisitEntriesInternal(cb);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -1009,8 +1121,8 @@ nsresult _OldStorage::AssembleCacheKey(nsIURI *aURI,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsAutoCString uriSpec;
-  if (aScheme.Equals(NS_LITERAL_CSTRING("http")) ||
-      aScheme.Equals(NS_LITERAL_CSTRING("https"))) {
+  if (aScheme.EqualsLiteral("http") ||
+      aScheme.EqualsLiteral("https")) {
     if (mLoadInfo->IsAnonymous()) {
       aCacheKey.AssignLiteral("anon&");
     }
@@ -1030,7 +1142,7 @@ nsresult _OldStorage::AssembleCacheKey(nsIURI *aURI,
       aCacheKey.AppendLiteral("uri=");
     }
   }
-  else if (aScheme.Equals(NS_LITERAL_CSTRING("wyciwyg"))) {
+  else if (aScheme.EqualsLiteral("wyciwyg")) {
     rv = aURI->GetSpec(uriSpec);
     NS_ENSURE_SUCCESS(rv, rv);
   }
