@@ -13,7 +13,7 @@
 class JSStreamWriter
 {
 public:
-  JSStreamWriter(std::ostream& aStream);
+  explicit JSStreamWriter(std::ostream& aStream);
   ~JSStreamWriter();
 
   void BeginObject();
@@ -22,6 +22,7 @@ public:
   void EndArray();
   void Name(const char *name);
   void Value(int value);
+  void Value(unsigned value);
   void Value(double value);
   void Value(const char *value, size_t valueLength);
   void Value(const char *value);
